import cv2
from sahi.prediction import ObjectPrediction, PredictionResult
from sahi.utils.cv import visualize_object_predictions
from yoloxdetect import YoloxDetector

from torchyolo.modelhub.basemodel import YoloDetectionModel


class YoloxDetectionModel(YoloDetectionModel):
    def load_model(self):
        model = YoloxDetector(self.model_path, config_path=self.config_path, device=self.device, hf_model=False)
        model.torchyolo = True
        model.conf = self.confidence_threshold
        model.iou = self.iou_threshold
        model.save = self.save
        model.show = self.show
        self.model = model

    def predict(self, image, yaml_file=None, tracker=False):
        object_prediction_list = []
        predict_list = self.model.predict(image_path=image, image_size=self.image_size)
        if tracker:
            return predict_list
        else:
            boxes, scores, cls_ids, class_names = predict_list[0], predict_list[1], predict_list[2], predict_list[3]
            for i in range(len(boxes)):
                box = boxes[i]
                category_id = int(cls_ids[i])
                score = scores[i]
                if score < self.confidence_threshold:
                    continue
                x0 = int(box[0])
                y0 = int(box[1])
                x1 = int(box[2])
                y1 = int(box[3])
                bbox = [x0, y0, x1, y1]
                category_name = class_names[category_id]

                object_prediction = ObjectPrediction(
                    bbox=bbox,
                    score=score,
                    category_id=category_id,
                    category_name=category_name,
                )
                object_prediction_list.append(object_prediction)

            prediction_result = PredictionResult(
                object_prediction_list=object_prediction_list,
                image=image,
            )
            if self.save:
                prediction_result.export_visuals(export_dir=self.save_path, file_name=self.output_file_name)

            if self.show:
                image = cv2.imread(image)
                output_image = visualize_object_predictions(image=image, object_prediction_list=object_prediction_list)
                cv2.imshow("Prediction", output_image["image"])
                cv2.waitKey(0)
                cv2.destroyAllWindows()

            return prediction_result



import cv2
from sahi.prediction import ObjectPrediction, PredictionResult
from sahi.utils.cv import visualize_object_predictions
from ultralytics import YOLO

from torchyolo.modelhub.basemodel import YoloDetectionModel


class Yolov8DetectionModel(YoloDetectionModel):
    def load_model(self):
        model = YOLO(self.model_path)
        model.conf = self.confidence_threshold
        model.iou = self.iou_threshold
        self.model = model

    def predict(self, image, yaml_file=None, tracker=False):
        prediction = self.model(image, imgsz=self.image_size)
        if tracker:
            return prediction
        else:     
            object_prediction_list = []
            for _, image_predictions_in_xyxy_format in enumerate(prediction):
                for pred in image_predictions_in_xyxy_format.cpu().detach().numpy():
                    x1, y1, x2, y2 = (
                        int(pred[0]),
                        int(pred[1]),
                        int(pred[2]),
                        int(pred[3]),
                    )
                    bbox = [x1, y1, x2, y2]
                    score = pred[4]
                    category_name = self.model.model.names[int(pred[5])]
                    category_id = pred[5]
                    object_prediction = ObjectPrediction(
                        bbox=bbox,
                        category_id=int(category_id),
                        score=score,
                        category_name=category_name,
                    )
                    object_prediction_list.append(object_prediction)

            prediction_result = PredictionResult(
                object_prediction_list=object_prediction_list,
                image=image,
            )
            if self.save:
                prediction_result.export_visuals(export_dir=self.save_path, file_name=self.output_file_name)

            if self.show:
                image = cv2.imread(image)
                output_image = visualize_object_predictions(image=image, object_prediction_list=object_prediction_list)
                cv2.imshow("Prediction", output_image["image"])
                cv2.waitKey(0)
                cv2.destroyAllWindows()

            return prediction_result



import cv2
from sahi.prediction import ObjectPrediction, PredictionResult
from sahi.utils.cv import visualize_object_predictions
from yolov6 import YOLOV6

from torchyolo.modelhub.basemodel import YoloDetectionModel


class Yolov6DetectionModel(YoloDetectionModel):
    def load_model(self):
        model = YOLOV6(self.model_path, device=self.device)
        model.torchyolo = True
        model.font_path = "torchyolo/configs/yolov6/Arial.ttf"
        model.conf = self.confidence_threshold
        model.iou = self.iou_threshold
        model.save_img = self.save
        model.show_img = self.show
        self.model = model

    def predict(self, image, yaml_file="torchyolo/configs/yolov6/coco.yaml",tracker=False):
        predictions, class_names = self.model.predict(source=image, img_size=self.image_size, yaml=yaml_file)
        if tracker:
            return predictions
        else:
            object_prediction_list = []
            for *xyxy, conf, cls in reversed(predictions.cpu().detach().numpy()):
                x1, y1, x2, y2 = (
                    int(xyxy[0]),
                    int(xyxy[1]),
                    int(xyxy[2]),
                    int(xyxy[3]),
                )
                bbox = [x1, y1, x2, y2]
                score = conf
                category_id = int(cls)
                category_name = class_names[category_id]

                object_prediction = ObjectPrediction(
                    bbox=bbox,
                    category_id=int(category_id),
                    score=score,
                    category_name=category_name,
                )
                object_prediction_list.append(object_prediction)

            prediction_result = PredictionResult(
                object_prediction_list=object_prediction_list,
                image=image,
            )
            if self.save:
                prediction_result.export_visuals(export_dir=self.save_path, file_name=self.output_file_name)

            if self.show:
                image = cv2.imread(image)
                output_image = visualize_object_predictions(image=image, object_prediction_list=object_prediction_list)
                cv2.imshow("Prediction", output_image["image"])
                cv2.waitKey(0)
                cv2.destroyAllWindows()

            return prediction_result
